Rem Antenna rotor controller hardware module

Rem This module is for the Prairie Digital Board
Rem See: "Inexpensive Automatic Control of Az-El Rotors"
Rem      Charles A. Richard, KM4EM, and James A. Sanford, WB4GCS,
Rem      The Amsat Journal, Vol 17, No. 6, November/December, 1994, p. 6

Rem I intend to make other modules for other boards eventually

Rem  CDGregory, K8CG, 6/15/95
Rem  Rev 6/21/95 - add constants for conversion of ADC to Az and El
Rem  Rev 7/5/95  - add 8255 initialization

Rem  Rev 7/6/96 - add ADC Type, initialization for all bits

Option Explicit
Option Compare Text

Global BaseAddress

Type ADC
    Channel As Integer
    Value As Integer
    PrevValue As Integer
    
    Gain As Single      ' conversion to volts
    Offset As Single
    Volts As Single
End Type

Rem Control bits for rotators
Global RotateUp As Integer
Global RotateDn As Integer
Global RotateCW As Integer
Global RotateCCW As Integer

Rem Control bits for step tuning
Const STEP_UP = &H10
Const STEP_DN = &H20

Sub AssignADCCal (AD As ADC)

Rem Initialize the ADC calibrations
AD.Gain = GetFloatingPref("PrairieBoard", "ADC" & Format$(AD.Channel) & "Gain", "0.0196")
AD.Offset = GetFloatingPref("PrairieBoard", "ADC" & Format$(AD.Channel) & "Offset", "0")

End Sub

Sub InitializePrairieParams ()
Rem Get preferences from .INI file
Rem Called at startup only

BaseAddress = GetIntegerPref("PrairieBoard", "BaseAddress", "772")

Az.ADC.Channel = GetIntegerPref("PrairieBoard", "AzChannel", "0")
El.ADC.Channel = GetIntegerPref("PrairieBoard", "ElChannel", "1")
AssignADCCal Az.ADC
AssignADCCal El.ADC

RotateUp = GetIntegerPref("PrairieBoard", "RotateUp", "16")
RotateDn = GetIntegerPref("PrairieBoard", "RotateDn", "32")
RotateCW = GetIntegerPref("PrairieBoard", "RotateCW", "64")
RotateCCW = GetIntegerPref("PrairieBoard", "RotateCCW", "128")

End Sub

Sub InitPrairieHdw ()
Rem Set up the 8255

frmControl!IOPortControl.IOPortAddress = BaseAddress + 3

Rem Set control word to 129 - Ports A/B out, Port C high out, low in
frmControl!IOPortControl.OutputValue = 129

End Sub

Sub PrairieTune (UpDown As String)
Dim PortCContents As Integer, DirectionBit As Integer, DirectionBitMask As Integer
Dim Counter As Long

Select Case UpDown
    Case "up"
        DirectionBit = STEP_UP
    Case "down"
        DirectionBit = STEP_DN
    Case Else
        DirectionBit = 0
End Select

DirectionBitMask = &HFF Xor (STEP_UP Or STEP_DN)

Rem Set the proper bit (pulse on)
    PortCContents = frmControl!IOPortC.InputValue
    frmControl!IOPortC.OutputValue = PortCContents Or DirectionBit

Rem Delay on my machine, 1-> 1msec pulses, 1000->4msec
    For Counter = 0 To DelayCount * DelayMsec
    Next Counter
Rem clear the bit (pulse off)
    PortCContents = frmControl!IOPortB.InputValue
    frmControl!IOPortC.OutputValue = PortCContents And DirectionBitMask
    For Counter = 0 To DelayCount * DelayMsec
    Next Counter

End Sub

Function ReadADC (Index As Integer) As Integer
Rem Read the ADC and return a number (0-255)
Rem Taken from PRRIEDRV.ASM ('getpos', 'getbyte' routines)

Dim Ibit As Integer
Dim Value As Integer
Dim ControlPortWord As Integer

Rem Hardware constants - derived from the comments
Rem Either the comments are wrong
Rem    or the control words are wrong !!

Const CS_High_CL_High = &HC0
Const CS_Low = &H40     ' ???
Const CL_Low = &H0
Const CL_High = &H40    ' ???
Const AD_Mask = &H8

Rem Set up the Port I/O controllers
frmControl!IOPortC.IOPortAddress = BaseAddress + 2
frmControl!IOPortC.InputMask = AD_Mask
frmControl!IOPortB.IOPortAddress = BaseAddress + 1
frmControl!IOPortB.InputMask = 255

Rem Select channels on mux ?
Rem this is supposed to select Azimuth without changing other bits
ControlPortWord = frmControl!IOPortB.InputValue And &HF8
frmControl!IOPortB.OutputValue = ControlPortWord

If Index = 1 Then
Rem While this selects Elevation
ControlPortWord = frmControl!IOPortB.InputValue Or 1
frmControl!IOPortB.OutputValue = ControlPortWord
End If

Rem Set up the board and read 8 serial bits ('getbyte')
Value = 0
frmControl!IOPortC.OutputValue = CS_High_CL_High
frmControl!IOPortC.OutputValue = CS_Low
For Ibit = 7 To 0 Step -1
    If frmControl!IOPortC.InputValue Then Value = Value + 2 ^ Ibit
    frmControl!IOPortC.OutputValue = CL_Low
    frmControl!IOPortC.OutputValue = CL_High
Next Ibit

Rem all done, return the value
ReadADC = Value

Rem for debug just return the setpoint (no hardware)
'ReadADC = frmControl!hscrManualControl(Index).Value

End Function

Sub RotateAzimuth (LeftRight As String, Mode As String)
Dim PortBContents As Integer, DirectionBit As Integer, DirectionBitMask As Integer

Select Case LeftRight
    Case "left"
        DirectionBit = RotateCCW
    Case "right"
        DirectionBit = RotateCW
    Case Else
        DirectionBit = 0
End Select

DirectionBitMask = &HFF Xor (RotateCCW Or RotateCW)

If Mode = "Start" Then
    PortBContents = frmControl!IOPortB.InputValue
    frmControl!IOPortB.OutputValue = PortBContents Or DirectionBit
Else 'end - always clear both bits!
    PortBContents = frmControl!IOPortB.InputValue
    frmControl!IOPortB.OutputValue = PortBContents And DirectionBitMask
End If

End Sub

Sub RotateElevation (UpDown As String, Mode As String)
Dim PortBContents As Integer, DirectionBit As Integer, DirectionBitMask As Integer

Select Case UpDown
    Case "down"
        DirectionBit = RotateDn
    Case "up"
        DirectionBit = RotateUp
    Case Else
        DirectionBit = 0
End Select

DirectionBitMask = &HFF Xor (RotateUp Or RotateDn)

If Mode = "Start" Then
    PortBContents = frmControl!IOPortB.InputValue
    frmControl!IOPortB.OutputValue = PortBContents Or DirectionBit
Else 'end - always clear both bits
    PortBContents = frmControl!IOPortB.InputValue
    frmControl!IOPortB.OutputValue = PortBContents And DirectionBitMask
End If

End Sub

