Rem DOPPLER.BAS
Rem K8CG 7/11/95


Option Explicit
Option Compare Text

Global StartFlag As Integer
Global NewDoppFactor As Double
Global TransponderInverting As Integer

Global DelayCount As Long, DelayMsec As Single

Global RadioCmd As String
Global NRadios As Integer

Type RadioData
    RadioName As String
    RigType As String
    VFOIndex As Integer
    DeviceAddress As Integer   ' Kenwood 790 or Icom
    LOOffsetFreq As Double
    Mode As String
    'ReverseFrequency  As Integer
    
    TuningType As String

    Rem Following frequencies are all in MHz
    DisplayFreq As Double
    InitialFreq As Double
    PowerOnFreq As Double
    ActualFreq As Double
    BandLimitLower As Double
    BandLimitUpper As Double
    'NominalFreq As Double
    TuneStep As Single

    Rem Following determine doppler compensation
    DoppFreq As Double      ' freq for doppler comp
    DoppFactor As Double    ' pseudo-velocity
    LinkDirection As Integer' comp on uplink or downlink
    CompType As String      ' string descr of dopp comp

    Rem Following are for serial-ascii-control
    StartupString As String
    ShutdownString As String
    StartupHexString As String
    ShutdownHexString As String
    CommPort As String
    CommPortID As Integer
    BaudRate As Single
End Type

Global Radio(1) As RadioData

Function FormatFrequency (Freq As Double) As String
Rem Standard display format for frequency

FormatFrequency = Format$(Freq, "######.000000")

End Function

Function FreqToBCD4 (Freq As Double) As String
Rem For Yaesu radios
Dim BCD As String, I As Integer, J As Integer, HexVal As Integer
Dim Temp As String, V  As String, GHz As Integer

Temp = Format$(Freq, "0000000000")
Temp = Left$(Temp, 9)' drop the units digit

Rem Take care of possible 1.2 GHz
If Left$(Temp, 2) = "12" Then GHz = True
Temp = Right$(Temp, 8)

For I = 0 To 3
HexVal = 0
    For J = 1 To 2
        V = Mid$(Temp, 2 * I + J, 1)
        If J = 1 Then
            HexVal = Val(V) * 16 + HexVal
        Else
            HexVal = Val(V) + HexVal
        End If
    Next J
    BCD = BCD + Chr$(HexVal)
Next I

If GHz Then  ' Yaesu
    Mid$(BCD, 1, 1) = Chr$(Asc(Mid$(BCD, 1, 1)) Mod 16 + 12 * 16)
End If

FreqToBCD4 = BCD

End Function

Function FreqToBCD5 (Freq As Double) As String
Rem For Icom radios
Dim BCD As String, I As Integer, J As Integer, HexVal As Integer
Dim Temp As String, V  As String, GHz As Integer

Rem                   0432100000 to nearest Hz  ?
Temp = Format$(Freq, "0000000000")

For I = 0 To 4 ' 5 BCD digits
HexVal = 0
    For J = 1 To 2
        V = Mid$(Temp, 2 * I + J, 1)
        If J = 1 Then
            HexVal = Val(V) * 16 + HexVal
        Else
            HexVal = Val(V) + HexVal
        End If
    Next J
    BCD = BCD + Chr$(HexVal)
Next I

FreqToBCD5 = BCD

End Function

Function GetFreqFromRadio (RadioIndex As Integer) As Double
Dim AcqFreq As String
Dim AcqResponse As String

Rem Fallback case
GetFreqFromRadio = Val(frmControl!lblNetFreq(RadioIndex))

Rem Try to get data from radio
        'RadioCmd = "IF;"   'first 14 bytes of FA/FB are same as IF
        Do
        Loop While ClearCommPort(Radio(RadioIndex).CommPortID)
        If Radio(RadioIndex).VFOIndex = 0 Then
            RadioCmd = "FA"
        Else
            RadioCmd = "FB"
        End If
        PutCommString Radio(RadioIndex).CommPortID, RadioCmd & ";"
        GetRadioResponse RadioIndex, AcqResponse

Rem Parse the response - must be an "IF" response
Rem If error or wrong response, beep and return
If InStr(AcqResponse, "?;") > 0 Or Not InStr(AcqResponse, RadioCmd) > 0 Then
    Beep
    MsgBox "Radio Interface Error " & AcqResponse, MB_OK, "WinPrairie"
    Exit Function
End If

If InStr(AcqResponse, "E;") > 0 Or InStr(AcqResponse, "O;") > 0 Then
    Beep
    MsgBox "Radio Interface Error " & AcqResponse, MB_OK, "WinPrairie"
    Exit Function
End If

AcqFreq = Mid$(AcqResponse, 3, 11)
Rem Assume the freq is in Hz, convert to MHz
GetFreqFromRadio = Val(AcqFreq) / 1000000#

End Function

Sub GetRadioResponse (RadioIndex As Integer, ResponseString As String)
Rem This function is called to get responses directly, bypassing the timer
Rem it returns a string terminated in ";"
Rem   Timer must be disabled by calling routine!
            
Dim Buffer As String, nChar As Integer

ResponseString = ""

    Do While InStr(ResponseString, ";") = 0
        Rem Now check the Comm port for input
        nChar = GetCommString(Radio(RadioIndex).CommPortID, Buffer)
        If nChar > 0 Then
            ResponseString = ResponseString & Buffer
        End If
    Loop

End Sub

Sub InitializeRadioParams (RadioIndex As Integer)
Dim RadioTopic As String

RadioTopic = "Radio" & Format$(RadioIndex)

Rem Obtain prefs from .INI file
Radio(RadioIndex).RadioName = GetStringPref(RadioTopic, "RadioName", RadioTopic)
Radio(RadioIndex).TuningType = GetStringPref(RadioTopic, "TuningType", "RS232Main")
Radio(RadioIndex).CompType = GetStringPref(RadioTopic, "CompType", "None")
Radio(RadioIndex).TuneStep = GetFloatingPref(RadioTopic, "TuneStep", "10")
DelayMsec = GetFloatingPref(RadioTopic, "DelayMsec", "1")              ' pulse width for up/down
Radio(RadioIndex).RigType = GetStringPref(RadioTopic, "RigType", "Kenwood")
Radio(RadioIndex).Mode = GetStringPref(RadioTopic, "Mode", "USB")
Radio(RadioIndex).VFOIndex = GetIntegerPref(RadioTopic, "VFOIndex", "0")
Radio(RadioIndex).DeviceAddress = GetIntegerPref(RadioTopic, "DeviceAddress", "16")
Radio(RadioIndex).StartupHexString = GetStringPref(RadioTopic, "StartupHexString", "")
Radio(RadioIndex).ShutdownHexString = GetStringPref(RadioTopic, "ShutdownHexString", "")
Radio(RadioIndex).StartupString = GetStringPref(RadioTopic, "StartupString", "")
Radio(RadioIndex).ShutdownString = GetStringPref(RadioTopic, "ShutdownString", "")
Radio(RadioIndex).LOOffsetFreq = GetFloatingPref(RadioTopic, "LOOffsetFrequency", "0")
Radio(RadioIndex).InitialFreq = GetFloatingPref(RadioTopic, "InitialFrequency", "21.28") 'MHz
Radio(RadioIndex).PowerOnFreq = GetFloatingPref(RadioTopic, "PowerOnFrequency", "21.28") 'MHz
Radio(RadioIndex).BandLimitLower = GetFloatingPref(RadioTopic, "BandLimitLower", "0.0") 'MHz
Radio(RadioIndex).BandLimitUpper = GetFloatingPref(RadioTopic, "BandLimitUpper", "10500.0") 'MHz
Radio(RadioIndex).CommPort = GetStringPref(RadioTopic, "CommPort", "COM3:")
Radio(RadioIndex).BaudRate = GetFloatingPref(RadioTopic, "BaudRate", "4800")

End Sub

Sub InitRig (RadioIndex As Integer)
Rem Initialize the rig
Dim Temp As Integer, Counter As Long

Radio(RadioIndex).DisplayFreq = Radio(RadioIndex).InitialFreq
Radio(RadioIndex).ActualFreq = Radio(RadioIndex).PowerOnFreq

Select Case Radio(RadioIndex).RigType
    Case "Kenwood"
        Rem Start with "clear buffer" then user string
        PutCommString Radio(RadioIndex).CommPortID, ";"
        If Radio(RadioIndex).TuningType = "RS232RIT" Then PutCommString Radio(RadioIndex).CommPortID, "RT1;"
        If Radio(RadioIndex).StartupString <> "" Then
            PutCommString Radio(RadioIndex).CommPortID, Radio(RadioIndex).StartupString
        Else
            PutCommString Radio(RadioIndex).CommPortID, MakeASCIIfromHex(Radio(RadioIndex).StartupHexString)
        End If
    Case "Icom"
        PutCommString Radio(RadioIndex).CommPortID, MakeASCIIfromHex(Radio(RadioIndex).StartupHexString)
    Case "Yaesu"
        Rem Start with "CAT ON", Full Dup, then user string
        PutCommString Radio(RadioIndex).CommPortID, Chr$(0) & Chr$(0) & Chr$(0) & Chr$(0) & Chr$(0)
        YaesuDelay
        PutCommString Radio(RadioIndex).CommPortID, Chr$(0) & Chr$(0) & Chr$(0) & Chr$(0) & Chr$(&HE)
        YaesuDelay
        If Radio(RadioIndex).StartupString <> "" Then
            PutCommString Radio(RadioIndex).CommPortID, Radio(RadioIndex).StartupString
        Else
            PutCommString Radio(RadioIndex).CommPortID, MakeASCIIfromHex(Radio(RadioIndex).StartupHexString)
        End If
        YaesuDelay
    Case Else
End Select

UpdateFreq RadioIndex

End Sub

Function MakeASCIIfromHex (HexString As String) As String
Rem Translate a hex string from the .INI file to ascii codes for
Rem    transmission to the radio

Dim Temp As String, Pointer As Integer, Nybble As Integer, ByteVal As Integer

Temp = ""
Pointer = 1
ByteVal = 0
Nybble = 2
If HexString <> "" Then
    Rem Loop to parse the input string
    While Pointer <= Len(HexString)
        Select Case Mid$(HexString, Pointer, 1)
            Case "0", "1", "2", "3", "4", "5", "6", "7", "8", "9"
                Nybble = Nybble - 1
                ByteVal = ByteVal + Val(Mid$(HexString, Pointer, 1)) * 16 ^ Nybble
            Case "A" To "F"
                Nybble = Nybble - 1
                ByteVal = ByteVal + (Asc(UCase$(Mid$(HexString, Pointer, 1))) - 64 + 9) * 16 ^ Nybble
            Case Else
                Rem skip anything else
        End Select
        If Nybble = 0 Then ' got two chars from input, make one in output
            Temp = Temp & Chr$(ByteVal)
            Nybble = 2
            ByteVal = 0
        End If
        Pointer = Pointer + 1
    Wend
End If

'Rem Debug
'For Pointer = 1 To Len(Temp)
'    Debug.Print Hex$(Asc(Mid$(Temp, Pointer, 1))); " ";
'Next Pointer
'Debug.Print

MakeASCIIfromHex = Temp

End Function

Sub MeasureCPUTiming ()
Dim I As Long, Temp As Double, Temp2 As Double

Rem Determine what the delay counter has to be:
Rem A delay of N * DelayCount will be N msec
Temp = CDbl(Now)
For I = 0 To 1000000   ' try 10^6 loops
Next I
Temp2 = CDbl(Now)
Rem Time difference is in Days.  Convert to milliseconds,
Rem     then get loops/msec (200 on my 486-33)
DelayCount = 1000000 / (86400000 * (Temp2 - Temp))


End Sub

Sub SetDeltaFreq (RadioIndex As Integer, OldFreq As Double, NewFreq As Double)
Rem This handles setting the frequency for radios that do not have
Rem   direct frequency commands
Dim Delta As Long, TuneString As String, I As Long

    Delta = (NewFreq - OldFreq) * 1000000# / Radio(RadioIndex).TuneStep
    If Delta > 0 Then ' down
        TuneString = "UP"
    Else ' up
        TuneString = "DOWN"
    End If
    
    Rem See if any action is really required
    If Delta = 0 Then Exit Sub
    
    Rem Otherwise, really tune the radio
    Screen.MousePointer = 11
    For I = 1 To Abs(Delta)
        Select Case Radio(RadioIndex).TuningType
            Case "Prairie"
                PrairieTune TuneString
            Case "Parallel"
                ParallelTune TuneString
            Case "RS232RIT"  ' kenwood only, and very crude!
                If TuneString = "UP" Then
                    RadioCmd = "RU;"
                Else
                    RadioCmd = "RD;"
                End If
                PutCommString Radio(RadioIndex).CommPortID, RadioCmd
            Case "KCT"
        End Select
    Next I
    Debug.Print RadioIndex; Left$(TuneString, 1); " "; Format$(OldFreq, "#######0.0000"); "-->"; Format$(NewFreq, "#######0.0000"); Abs(Delta)
    Screen.MousePointer = 0

End Sub

Sub SetMainFreq (RadioIndex As Integer, Freq As Double)
Rem Set the main frequency via RS232

Dim FreqString As String

Select Case Radio(RadioIndex).RigType
    Case "Kenwood"
        Rem determine which VFO to send info to
        RadioCmd = "FA"
        If Radio(RadioIndex).VFOIndex = 1 Then RadioCmd = "FB"
        Rem Abs func here takes care of high-side LO injection
        FreqString = Format$(Abs(Freq) * 1000000#, "00000000000")
        RadioCmd = RadioCmd & FreqString & ";"
        PutCommString Radio(RadioIndex).CommPortID, RadioCmd
    Case "Yaesu"
        Rem For the FT-736
        Rem determine which VFO to send info to
        RadioCmd = Chr$(&H1E) ' Receive VFO
        If Radio(RadioIndex).VFOIndex = 1 Then RadioCmd = Chr$(&H2E) ' Tx VFO
        Rem Abs func here takes care of high-side LO injection
        FreqString = FreqToBCD4(Abs(Freq) * 1000000#)
        PutCommString Radio(RadioIndex).CommPortID, FreqString & RadioCmd
        YaesuDelay
    Case "Icom"  ' based on the IC-275/475/271/275 for now
        RadioCmd = Chr$(&HFE) & Chr$(&HFE) & Chr$(Radio(RadioIndex).DeviceAddress) & Chr$(&HE)
        FreqString = Chr$(&H5) & FreqToBCD5(Abs(Freq) * 1000000#) & Chr$(&HFD)
        PutCommString Radio(RadioIndex).CommPortID, RadioCmd
    Case Else
End Select

End Sub

Sub ShutdownRig (RadioIndex As Integer)
Rem Shutdown the rig
Rem Send user string first, then codes to shutdown radio

Dim Counter As Long

Rem 6/22/96 - only applicable if there is an RS232 connection
If Radio(RadioIndex).TuningType = "RS232Main" Or Radio(RadioIndex).TuningType = "RS232RIT" Then
    Select Case Radio(RadioIndex).RigType
        Case "Kenwood"
            PutCommString Radio(RadioIndex).CommPortID, ";"
            If Radio(RadioIndex).StartupString <> "" Then
                PutCommString Radio(RadioIndex).CommPortID, Radio(RadioIndex).ShutdownString
            Else
                PutCommString Radio(RadioIndex).CommPortID, MakeASCIIfromHex(Radio(RadioIndex).ShutdownHexString)
            End If
        Case "Icom"
            PutCommString Radio(RadioIndex).CommPortID, MakeASCIIfromHex(Radio(RadioIndex).ShutdownHexString)
        Case "Yaesu"
            If Radio(RadioIndex).StartupString <> "" Then
                PutCommString Radio(RadioIndex).CommPortID, Radio(RadioIndex).ShutdownString
            Else
                PutCommString Radio(RadioIndex).CommPortID, MakeASCIIfromHex(Radio(RadioIndex).ShutdownHexString)
            End If
            YaesuDelay
            ' Full dup off, then CAT off
            PutCommString Radio(RadioIndex).CommPortID, Chr$(0) & Chr$(0) & Chr$(0) & Chr$(0) & Chr$(&H8E)
            YaesuDelay
            PutCommString Radio(RadioIndex).CommPortID, Chr$(0) & Chr$(0) & Chr$(0) & Chr$(0) & Chr$(&H80)
            YaesuDelay
        Case Else
    End Select
Else
    Rem Non-RS232 types.  Return to Power-on Frequency
    Radio(RadioIndex).DisplayFreq = Radio(RadioIndex).PowerOnFreq
    Radio(RadioIndex).LOOffsetFreq = 0  ' cancel any offsets
    Radio(RadioIndex).DoppFreq = 0
    UpdateFreq RadioIndex
End If

End Sub

Sub UpdateFreq (RadioIndex As Integer)
Dim TuneString As String
Dim NewFreq As Double, TargetFreq  As Double, Doppler As Double

If Radio(RadioIndex).DisplayFreq = 0 Then Exit Sub

Rem Compute new physical rig frequency
    NewFreq = Radio(RadioIndex).DisplayFreq - Radio(RadioIndex).LOOffsetFreq
    Doppler = NewDoppFactor * Radio(RadioIndex).DoppFreq * Radio(RadioIndex).LinkDirection
    NewFreq = NewFreq + Doppler / 1000000#
    NewFreq = Abs(NewFreq)

Rem Test for band limits
If NewFreq < Radio(RadioIndex).BandLimitLower Or NewFreq > Radio(RadioIndex).BandLimitUpper Then Exit Sub

Rem Round-off to closest available frequency
    TargetFreq = (Radio(RadioIndex).TuneStep / 1000000#) * Int(.5 + NewFreq * 1000000# / Radio(RadioIndex).TuneStep)

Rem See if a change can be made (must be > 1 Hz)
    If Abs(TargetFreq - Radio(RadioIndex).ActualFreq) < .000001 Then Exit Sub

If Radio(RadioIndex).TuningType = "RS232MAIN" Then
    Rem Direct main tuning is different from all other cases
    SetMainFreq RadioIndex, TargetFreq
Else
    If Abs(TargetFreq - Radio(RadioIndex).ActualFreq) > .003 And Radio(RadioIndex).TuningType = "RS232RIT" Then ' more than 3kHz
        Rem Direct main tuning is possible in this case  ?
        SetMainFreq RadioIndex, TargetFreq
    Else
        Rem All other cases are "delta" tuning
        Rem "Prairie", "Parallel", "KCT", etc.
        SetDeltaFreq RadioIndex, Radio(RadioIndex).ActualFreq, TargetFreq
    End If
End If

Rem Update the radio data
Radio(RadioIndex).ActualFreq = TargetFreq
frmControl.lblNetFreq(RadioIndex) = FormatFrequency(Radio(RadioIndex).DisplayFreq)
frmControl!lblDoppler(RadioIndex) = Format(Doppler, "#####0")

End Sub

Sub UpdateOtherFreq (RadioIndex As Integer, Delta As Double)
Rem Update the Other radio channel
Rem Used if frequencies are tracking

Dim OtherIndex As Integer, OtherFreq As Double

If RadioIndex = 1 Then
    OtherIndex = 0
Else
    OtherIndex = 1
End If

OtherFreq = Val(frmControl!lblNetFreq(OtherIndex))
Radio(OtherIndex).DisplayFreq = OtherFreq + Delta
UpdateFreq OtherIndex

End Sub

Sub YaesuDelay ()
Rem Delay for Yaesu FT736 to avoid lockups
Rem Nominally 100msec delay ?

Dim Counter As Long

    For Counter = 0 To DelayCount * 500
    Next Counter

End Sub

